/**
 * @fileoverview Shader uniforms module.
 */
import { UniformType, Viewport } from './gl';
import { Cube, CubeFace, CubeOr, Matrix3, Matrix4, Vector3, Vector4 } from './math';
declare type IsOrReturns<T> = T | (() => T);
/**
 * Types of data that uniforms accept.
 */
export declare type UniformData = number | Float32List | CubeOr<TexImageSource>;
/**
 * Uniform base class. All uniforms are a superclass of this
 * class.
 */
export declare abstract class Uniform<Data extends UniformData> {
    protected readonly type_: UniformType;
    readonly name: string;
    protected dataOrCb_?: IsOrReturns<Data>;
    constructor(type_: UniformType, name: string, dataOrCb_?: IsOrReturns<Data>);
    static checkType(u: Uniform<UniformData>, wantType: UniformType): void;
    static dataOrCallback<Data extends UniformData>(u: Uniform<Data>): IsOrReturns<Data>;
    set(dataOrCb: IsOrReturns<Data>): this;
    data(): Data;
    send(gl: WebGLRenderingContext, program: WebGLProgram): void;
}
declare class BytesUniform extends Uniform<number> {
    send(gl: WebGLRenderingContext, program: WebGLProgram): void;
}
/**
 * Send a boolean uniform to a shader.
 */
export declare const BooleanUniform: (name: string, data?: IsOrReturns<number>) => BytesUniform;
/**
 * Send a float uniform to a shader.
 */
export declare const FloatUniform: (name: string, data?: IsOrReturns<number>) => BytesUniform;
/**
 * Send a integer uniform to a shader.
 */
export declare const IntegerUniform: (name: string, data?: IsOrReturns<number>) => BytesUniform;
declare type SequenceUniformType = UniformType.VECTOR | UniformType.MATRIX;
declare class SequenceUniform extends Uniform<Float32List> {
    readonly dimension: number;
    constructor(type: SequenceUniformType, name: string, dimension: number, dataOrCb?: IsOrReturns<Float32List>);
    private validateData_;
    send(gl: WebGLRenderingContext, program: WebGLProgram): void;
    set(dataOrCb: IsOrReturns<Float32List>): this;
    static checkDimension(u: SequenceUniform, wantDimension: number): void;
}
declare type SequenceUniformBuilder = (name: string, data?: IsOrReturns<Float32List>) => SequenceUniform;
/**
 * Sends a 2-dimensional vector to a shader.
 */
export declare const Vec2Uniform: SequenceUniformBuilder;
/**
 * Sends a 3-dimensional vector to a shader.
 */
export declare const Vec3Uniform: SequenceUniformBuilder;
/**
 * Sends a 4-dimensional vector to a shader.
 */
export declare const Vec4Uniform: SequenceUniformBuilder;
/**
 * Sends a 2-dimensional matrix to a shader.
 */
export declare const Mat2Uniform: SequenceUniformBuilder;
/**
 * Sends a 3-dimensional matrix to a shader.
 */
export declare const Mat3Uniform: SequenceUniformBuilder;
/**
 * Sends a 4-dimensional matrix to a shader.
 */
export declare const Mat4Uniform: SequenceUniformBuilder;
interface ModelMatOptions {
    scale?: number | Vector3;
    rotate?: Vector4;
    translate?: Vector3;
}
declare class ModelMatUniformImpl extends SequenceUniform {
    private scaleMatrix_;
    private rotationMatrix_;
    private translation_;
    constructor(name: string, opts: ModelMatOptions);
    scaleMatrix(): Matrix4;
    rotationMatrix(): Matrix3;
    private rotationMatrix4_;
    translation(): Vector3;
    matrix(): Matrix4;
    scale(...args: number[]): void;
    setScale(...args: number[]): void;
    rotate(theta: number, ...axis: Vector3): void;
    setRotate(theta: number, ...axis: Vector3): void;
    translate(...args: Vector3): void;
    setTranslation(...args: Vector3): void;
}
/**
 * Sends a model matrix to a shader as a uniform that applies
 * a scale, rotation, and translation (in that order).
 */
export declare const ModelMatUniform: (name: string, opts?: ModelMatOptions) => ModelMatUniformImpl;
/**
 * Sends the resulting normal matrix for a given
 * model matrix to a shader.
 *
 * The normal matrix is the inverse transpose of
 * the rotation component of the model matrix.
 * Since for any rotation matrix R, it's inverse is
 * R^T.
 */
export declare const NormalMatUniform: (name: string, modelMat: ModelMatUniformImpl) => SequenceUniform;
/**
 * View matrix uniform with accessor and mutator methods for
 * the
 */
export declare class ViewMatUniformImpl extends SequenceUniform {
    private eye_;
    private at_;
    private up_;
    constructor(name: string, eye_: Vector3, at_: Vector3, up_: Vector3);
    private static validateVector3;
    eye(): Vector3;
    at(): Vector3;
    up(): Vector3;
    setEye(...eye: Vector3): void;
    setAt(...at: Vector3): void;
    setUp(...up: Vector3): void;
}
/**
 * Sends a view matrix uniform for the given
 * eye, at, and up vectors.
 */
export declare const ViewMatUniform: (name: string, eye: Vector3, at: Vector3, up: Vector3) => ViewMatUniformImpl;
declare class PerspectiveMatUniformImpl extends SequenceUniform {
    private fovy_;
    private aspect_;
    private near_;
    private far_;
    constructor(name: string, fovy_: number, aspect_: number, near_: number, far_: number | null);
    fovy(): number;
    aspect(): number;
    near(): number;
    far(): number;
    setFovy(fovy: number): void;
    setAspect(aspect: number): void;
    setNear(near: number): void;
    setFar(far: number): void;
}
/**
 * Sends a perspective 4D matrix uniform.
 */
export declare const PerspectiveMatUniform: (name: string, fovy: number, aspect: number, near: number, far?: number | null) => PerspectiveMatUniformImpl;
/**
 * Possible types to use as texture data sources.
 */
export declare type TextureData = CubeOr<TexImageSource>;
interface Texture2DBuffers {
    frameBuffer: WebGLFramebuffer;
    renderBuffer: WebGLRenderbuffer;
}
declare type CubeTextureBuffers = {
    [k in keyof Texture2DBuffers]: Cube<Texture2DBuffers[k]>;
};
declare type TextureBuffers = Texture2DBuffers | CubeTextureBuffers;
export declare class TextureUniform<Data extends TextureData> extends Uniform<Data> {
    protected offset_: number;
    protected texture_: WebGLTexture;
    protected textureBuffers_: TextureBuffers;
    protected shouldBuildTextureFromData_(): boolean;
    prepare(gl: WebGLRenderingContext, p: WebGLProgram): void;
    buffers(gl: WebGLRenderingContext, v: Viewport): TextureBuffers;
    set(dataOrCb: IsOrReturns<Data>): this;
}
/**
 * Tests if a uniform is for a texture.
 */
export declare const isTextureUniform: (u: Uniform<UniformData>) => u is TextureUniform<CubeOr<TexImageSource>>;
/**
 * Implementation of a 2D texture uniform.
 */
export declare class Texture2DUniformImpl extends TextureUniform<TexImageSource> {
    protected textureBuffers_: Texture2DBuffers;
    constructor(name: string, dataOrCb?: IsOrReturns<TexImageSource>);
    prepare(gl: WebGLRenderingContext, program: WebGLProgram): void;
    send(gl: WebGLRenderingContext, program: WebGLProgram): void;
    buffers(gl: WebGLRenderingContext, viewport: Viewport): Texture2DBuffers;
}
/**
 * Sends a 2D texture uniform to a shader.
 */
export declare const Texture2DUniform: (name: string, data?: IsOrReturns<TexImageSource>) => Texture2DUniformImpl;
export declare class CubeTextureImpl extends TextureUniform<Cube<TexImageSource>> {
    protected textureBuffers_: CubeTextureBuffers;
    private validateData;
    set(dataOrCb: IsOrReturns<Cube<TexImageSource>>): this;
    prepare(gl: WebGLRenderingContext, program: WebGLProgram): void;
    send(gl: WebGLRenderingContext, program: WebGLProgram): void;
    buffers(gl: WebGLRenderingContext, viewport: Viewport): CubeTextureBuffers;
}
/**
 * Sends a cube texture uniform to a shader.
 */
export declare const CubeTextureUniform: (name: string, data?: IsOrReturns<Cube<TexImageSource>>) => CubeTextureImpl;
/**
 * Tests if a uniform is a cube texture uniform.
 */
export declare const isCubeTextureUniform: (u: TextureUniform<CubeOr<TexImageSource>>) => u is CubeTextureImpl;
/**
 * Implementation of a cube camera uniform.
 */
export declare class CubeCameraUniformImpl extends CubeTextureImpl {
    private position_;
    private viewMat_;
    private perspectiveMat_;
    private static upVectors;
    private static atVectors;
    constructor(name: string, position_: Vector3, viewMat_: ViewMatUniformImpl, perspectiveMat_: PerspectiveMatUniformImpl);
    set(dataOrCb: IsOrReturns<Cube<TexImageSource>>): never;
    render(renderShader: (cf: CubeFace) => void): void;
}
/**
 * Sends a cube texture to a shader rendered from
 * a cube camera.
 */
export declare const CubeCameraUniform: (name: string, position?: Vector3, viewMat?: ViewMatUniformImpl, perspectiveMat?: PerspectiveMatUniformImpl) => CubeCameraUniformImpl;
export {};
